import asyncio

# 兼容windows系统
import sys
if sys.platform == "win32":
    from asyncio.windows_events import WindowsSelectorEventLoopPolicy
    asyncio.set_event_loop_policy(WindowsSelectorEventLoopPolicy())

from literaturescrape.core.models import Response, Literature
from literaturescrape.sources import BaseScraper, OpenAlexScraper, ThirdironScraper


scraper_list: list[BaseScraper] = [OpenAlexScraper(), ThirdironScraper()]

async def async_summary(doi: str) -> dict | None:
    tasks = [asyncio.create_task(scraper.execute(doi)) for scraper in scraper_list]
    results: list[Response | None] = await asyncio.gather(*tasks)
    
    # 过滤失败的请求
    succeed_results:list[Response] = [item for item in results if item is not None and item.literature is not None]
    ret = sorted(succeed_results, key=lambda x: x.priority, reverse=True)
    # todo: 选择最佳匹配
    if ret:
        literature:Literature = ret[0].literature # type: ignore
        return literature.to_dict()
    else:
        return None


def sync_summary(doi: str) -> dict | None:
    try:
        loop = asyncio.get_running_loop()
    except RuntimeError:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
    info = loop.run_until_complete(async_summary(doi))
    return info





import os
from loguru import logger
from urllib.parse import quote
from curl_cffi.requests import AsyncSession
from literaturescrape.core.models import Literature

from literaturescrape.sources import BaseScraper


class ThirdironScraper(BaseScraper):
    def __init__(self):
        self.name = "thirdiron"
        self.priority = 9

    async def _fetch(self, doi: str):
        async with AsyncSession(impersonate='chrome119', verify=False, timeout=15) as client:
            query = quote(f"doi:{doi}", safe='')
            url = f"https://api.thirdiron.com/v2/articles/{query}?include=issue%2Cjournal&reload=true"
            resp = await client.get(url, headers={'Authorization': 'Bearer 190f1a5d-5685-4b77-ba52-ee25d697542e'})
            ret = resp.json()
            return ret
    
    def _extract(self, data: dict):
        additional_data = {}
        
        included = data.get("included", [])
        data = data["data"]["attributes"]
        
        try:
            start_page = data.get("startPage", '').strip()
            end_page = data.get("endPage", '').strip()
            pages = f"{start_page}-{end_page}" if start_page and end_page else start_page
            author = ", ".join([" ".join(au.split(", ", 1)[::-1]) for au in data["authors"].split("; ")])
            
            additional_data["pages"] = pages
            additional_data["author"] = author
        except:
            pass
        
        
        for item in included:
            if item["type"] == "issues":
                additional_data["volume"] = item['volume']
                additional_data["issue"] = item['number']
                
            elif item["type"] == "journals": 
                additional_data["jour_name"] = item['attributes']['title']
                
                issn = item['attributes']['issn']
                if issn:
                    additional_data["jour_issn"] = issn[:4] + "-" + issn[4:]
                
        
        return Literature(
            title=data["title"],
            doi=data["doi"],
            pmid=data["pmid"],
            jour_name=additional_data.get("jour_name"),
            jour_issn=additional_data.get("jour_issn"),
            openAccess=data["openAccess"],
            publish_date=data["date"],
            year=data["date"][:4],
            volume=additional_data.get("volume"),
            issue=additional_data.get("issue"),
            page=additional_data.get("pages"),
            abstract=data["abstract"],
            author=additional_data.get("author"),
        )
    
    
if __name__ == "__main__":
    import asyncio
    scraper = ThirdironScraper()
    ret = asyncio.run(scraper.execute("10.1186/s12967-024-05786-4"))
    print(ret)


import os
from loguru import logger
from curl_cffi.requests import AsyncSession
from core.models import Literature

from sources import BaseScraper


class PubmedScraper(BaseScraper):
    def __init__(self):
        self.name = "pubmed"
        self.priority = 5
        
        
    async def _fetch_pmid(self, doi: str) -> str | None:
        async with AsyncSession(impersonate='chrome119', verify=False, timeout=15) as client:
            url = f"https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?db=pubmed&term={doi}&retmode=json&field=doi"
            resp = await client.get(url)
            resp.raise_for_status()
            ret = resp.json()
            assert ret['esearchresult']['count'] == 1, f"DOI {doi} not found in Pubmed"
            return ret['esearchresult']['idlist'][0]
        
        
    async def _fetch_info(self, pmid: str):
        async with AsyncSession(impersonate='chrome119', verify=False, timeout=15) as client:
            resp = await client.get(f"https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=pubmed&id={pmid}")
            resp.raise_for_status()
            return resp.json()

    async def _fetch(self, doi: str):
        try:
            pmid = await self._fetch_pmid(doi)
            if pmid is not None:
                return await self._fetch_info(pmid)
        except Exception as e:
            logger.error(f"Error fetching pmid for {doi}: {e}")
            return None
    
    def _extract(self, data: dict):
        biblio = data.get("biblio") or dict()

        pages = [str(page) for page in [biblio.get("first_page"), biblio.get("last_page")] if page is not None]
        if len(pages)==2 and pages[0]==pages[1]:
            pages = [pages[0]]
        pages = " - ".join(pages)

        absLst = data.get("abstract_inverted_index")
        if absLst:
            absMap = {}
            for value, inx in absLst.items():
                absMap.update({key:value for key in inx})
            abstract = " ".join([word[1] for word in sorted(absMap.items(), key=lambda item: item[0])])
        else:
            abstract = ""

        doi = data.get("ids", {}).get("doi", "").removeprefix('https://doi.org/')

        additional_info = {}
        
        try:
            pmid = data.get("ids", {}).get("pmid")
            if pmid:
                additional_info["pmid"] = pmid.removeprefix('https://pubmed.ncbi.nlm.nih.gov/')
            
            additional_info["journal_name"] = data.get("primary_location", {}).get("source", {}).get("display_name")
            additional_info["jour_issn"] = data.get("primary_location", {}).get("source", {}).get("issn_l")
            additional_info["is_oa"] = data.get("open_access", {}).get("is_oa")
            additional_info["author"] = ", ".join([au["author"]["display_name"] for au in data.get("authorships", [])])
        except:
            # 这些附加数据, 失败了就不管了
            pass
        

        return Literature(
            title=data["title"],
            doi=doi,
            pmid=additional_info.get("pmid"),
            jour_name=additional_info.get("journal_name"),
            jour_issn=additional_info.get("jour_issn"),
            openAccess=additional_info.get("is_oa", False),
            publish_date=data.get("publication_date"),
            year=data.get("publication_year"),
            volume=biblio.get("volume") or "",
            issue=biblio.get("issue") or "",
            page=pages,
            abstract=abstract,
            author=additional_info.get("author"),
        )
    
    
if __name__ == "__main__":
    import asyncio
    scraper = PubmedScraper()
    ret = asyncio.run(scraper.execute("10.1186/s12967-024-05786-4"))
    print(ret)



import asyncio
from loguru import logger

from abc import ABC, abstractmethod
from literaturescrape.core.models import Literature, Response
from typing import Optional


class BaseScraper(ABC):
    def __init__(self, name:str):
        self.name = name
        self.priority = 0  # 优先级
    
    @abstractmethod
    async def _fetch(self, doi:str) -> str:
        pass
    
    @abstractmethod
    def _extract(self, data:str) -> Optional[Literature]:
        pass
    
    async def execute(self, doi:str) -> Optional[Response]:
        response = Response(source=self.name)
        
        data = None
        
        try:
            data = await self._fetch(doi)
        except Exception as e:
            logger.debug(f"[{self.name}] [{doi}] {e}")
            response.fetch_flag = False
            response.fetch_err_reason = str(e)
        else:
            response.fetch_flag = True
        
        if data:
            try:
                literature = await asyncio.to_thread(self._extract, data)
            except Exception as e:
                logger.debug(f"[{self.name}] [{doi}] {e}")
                response.extract_flag = False
                response.extract_err_reason = str(e)
            else:
                response.extract_flag = True
                
                if isinstance(literature, Literature):
                    response.finished = True
                    
                response.literature = literature
        
        response.priority = self.priority
                
        return response


import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="literaturescrape",
    version="0.250205.4",
    author="zzzzls",
    author_email="245129129@qq.com",
    description="Get literature information",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zzzzls/",
    packages=setuptools.find_packages(),
    license='MIT',
    keywords=['literature', 'paper', 'doi', 'abstract'],
    install_requires=[
        "curl_cffi>=0.6.2",
        "loguru>=0.7.2"
    ]
)



from dataclasses import dataclass, asdict
from typing import Optional

@dataclass
class Literature:
    title: str
    doi: Optional[str] = None
    IF: Optional[str] = None
    pmid: Optional[str] = None
    jour_name: Optional[str] = None
    jour_issn: Optional[str] = None
    
    openAccess: bool = False
    
    publish_date: Optional[str] = None
    year: Optional[str] = None
    volume: Optional[str] = None
    issue: Optional[str] = None
    page: Optional[str] = None
    
    author: Optional[str] = None
    abstract: Optional[str] = None
    
    def to_dict(self):
        return asdict(self)
    
    
@dataclass
class Response:
    source: str
    priority: int = 0
    
    fetch_flag: bool = False
    extract_flag: bool = False
    fetch_err_reason: Optional[str] = None
    extract_err_reason: Optional[str] = None
    
    finished: bool = False
    literature: Optional[Literature] = None

